import pytest
from common import (
    DOCS_SCHEMAS_PATH,
    MANIFEST_ID_PREFIX,
    MANIFESTS_PATH,
    METASCHEMA_ID,
    SCHEMA_ID_PREFIX,
    SCHEMAS_PATH,
    VALID_SCHEMA_FILENAME_RE,
    YAML_SCHEMA_PATH,
    assert_yaml_header_and_footer,
    is_deprecated,
    list_description_ids,
    list_example_ids,
    list_latest_schema_paths,
    list_legacy_schema_paths,
    list_refs,
    list_schema_paths,
    load_yaml,
    path_to_id,
    path_to_tag,
    ref_to_id,
    split_id,
)
from jsonschema.validators import Draft4Validator, RefResolver


@pytest.fixture(scope="session")
def schemas():
    return [load_yaml(p) for p in list_schema_paths(SCHEMAS_PATH)]


@pytest.fixture(scope="session")
def manifests():
    return [load_yaml(p) for p in list_schema_paths(MANIFESTS_PATH)]


@pytest.fixture(scope="session")
def yaml_schemas():
    return [load_yaml(p) for p in list_schema_paths(YAML_SCHEMA_PATH)]


@pytest.fixture(scope="session")
def latest_schemas():
    return [load_yaml(p) for p in list_latest_schema_paths(SCHEMAS_PATH)]


@pytest.fixture(scope="session")
def legacy_schemas():
    return [load_yaml(p) for p in list_legacy_schema_paths(SCHEMAS_PATH)]


def get_schema_ids(schemas):
    result = set()
    for schema in schemas:
        if "id" in schema:
            result.add(schema["id"])
    return result


@pytest.fixture(scope="session")
def latest_schema_ids(latest_schemas):
    return get_schema_ids(latest_schemas)


@pytest.fixture(scope="session")
def legacy_schema_ids(legacy_schemas):
    return get_schema_ids(legacy_schemas)


@pytest.fixture(scope="session")
def manifest_ids(manifests):
    return get_schema_ids(manifests)


def add_schemas(path, prefix, result):
    with open(path) as f:
        content = f.read()

    lines = content.split("\n")
    i = 0
    while i < len(lines):
        if lines[i].startswith(".. asdf-autoschemas::"):
            i += 1
            while i < len(lines) and (lines[i].strip() == "" or lines[i].startswith(" ")):
                possible_id = lines[i].strip()
                if len(possible_id) > 0:
                    result.append(f"{prefix}{possible_id}")
                i += 1
        else:
            i += 1


@pytest.fixture(scope="session")
def docs_schema_ids():
    result = []
    for path in DOCS_SCHEMAS_PATH.glob("**/*.rst"):
        if path != DOCS_SCHEMAS_PATH / "manifest.rst":
            add_schemas(path, SCHEMA_ID_PREFIX, result)
    return result


@pytest.fixture(scope="session")
def docs_legacy_schema_ids():
    result = []
    for path in DOCS_SCHEMAS_PATH.glob("**/legacy.rst"):
        add_schemas(path, SCHEMA_ID_PREFIX, result)
    return result


@pytest.fixture(scope="session")
def docs_manifest_ids():
    result = []
    for path in DOCS_SCHEMAS_PATH.glob("**/manifest.rst"):
        add_schemas(path, MANIFEST_ID_PREFIX, result)
    return result


def _get_tag(schema):
    if "tag" in schema:
        return schema["tag"]
    elif "anyOf" in schema:
        for elem in schema["anyOf"]:
            if "tag" in elem:
                return elem["tag"]
    return None


@pytest.fixture(scope="session")
def latest_schema_tags(latest_schemas):
    result = set()
    for schema in latest_schemas:
        tag = _get_tag(schema)
        if tag is not None:
            result.add(tag)
    return result


@pytest.fixture(scope="session")
def schema_tags(schemas):
    result = set()
    for schema in schemas:
        tag = _get_tag(schema)
        if tag is not None:
            result.add(tag)
    return result


@pytest.fixture(scope="session")
def tag_to_schema(schemas):
    result = {}
    for schema in schemas:
        tag = _get_tag(schema)
        if tag is not None:
            if tag not in result:
                result[tag] = []
            result[tag].append(schema)
    return result


@pytest.fixture(scope="session")
def id_to_schema(schemas):
    result = {}
    for schema in schemas:
        if "id" in schema:
            if schema["id"] not in result:
                result[schema["id"]] = []
            result[schema["id"]].append(schema)
    return result


@pytest.fixture(scope="session")
def assert_schema_correct(tag_to_schema, id_to_schema):
    def _assert_schema_correct(path):
        __tracebackhide__ = True

        assert VALID_SCHEMA_FILENAME_RE.match(path.name) is not None, f"{path.name} is an invalid schema filename"

        assert_yaml_header_and_footer(path)

        schema = load_yaml(path)

        assert "$schema" in schema, f"{path.name} is missing $schema key"
        assert schema["$schema"] == METASCHEMA_ID, f"{path.name} has wrong $schema value (expected {METASCHEMA_ID})"

        expected_id = path_to_id(path)
        expected_tag = path_to_tag(path)

        assert "id" in schema, f"{path.name} is missing id key (expected {expected_id})"
        assert schema["id"] == expected_id, f"{path.name} id doesn't match filename (expected {expected_id})"

        if "tag" in schema:
            assert schema["tag"] == expected_tag, f"{path.name} tag doesn't match filename (expected {expected_tag})"

        assert "title" in schema, f"{path.name} is missing title key"
        assert len(schema["title"].strip()) > 0, f"{path.name} title must have content"

        assert "description" in schema, f"{path.name} is missing description key"
        assert len(schema["description"].strip()) > 0, f"{path.name} description must have content"

        assert len(id_to_schema[schema["id"]]) == 1, f"{path.name} does not have a unique id"

        if "tag" in schema:
            assert len(tag_to_schema[schema["tag"]]) == 1, f"{path.name} does not have a unique tag"

        id_base, _ = split_id(schema["id"])
        for example_id in list_example_ids(schema):
            example_id_base, _ = split_id(example_id)
            if example_id_base == id_base and example_id != schema["id"]:
                assert False, f"{path.name} contains an example with an outdated tag"

        for description_id in list_description_ids(schema):
            if len(description_id.rsplit("-", 1)) > 1:
                description_id_base, _ = split_id(description_id)
                if description_id_base == id_base and description_id != schema["id"]:
                    assert False, f"{path.name} descriptioon contains an outdated ref"

    return _assert_schema_correct


@pytest.fixture(scope="session")
def assert_latest_schema_correct(latest_schema_ids):
    def _assert_latest_schema_correct(path):
        __tracebackhide__ = True

        schema = load_yaml(path)

        if is_deprecated(schema["id"]):
            return

        refs = [r.split("#")[0] for r in list_refs(schema) if not r.startswith("#") and not r == METASCHEMA_ID]
        for ref in refs:
            ref_id = ref_to_id(schema["id"], ref)
            assert ref_id in latest_schema_ids, (
                f"{path.name} is the latest version of a schema, " f"but references {ref}, which is not latest"
            )

        for example_id in list_example_ids(schema):
            assert example_id in latest_schema_ids, (
                f"{path.name} is the latest version of a schema, "
                f"but its examples include {example_id}, which is not latest"
            )

        for description_id in list_description_ids(schema):
            if len(description_id.rsplit("-", 1)) > 1:
                assert description_id in latest_schema_ids, (
                    f"{path.name} is the latest version of a schema, "
                    f"but its description includes a ref to {description_id}, "
                    "which is not latest"
                )

    return _assert_latest_schema_correct


@pytest.fixture(scope="session")
def create_validator(schemas, yaml_schemas):
    """
    Fixture method that creates validators with access to schemas in
    this repository.
    """

    def _create_validator(schema):
        store = {s["id"]: s for s in schemas + yaml_schemas}

        resolver = RefResolver.from_schema(schema, id_of=Draft4Validator.ID_OF, store=store)

        return Draft4Validator(schema, resolver=resolver)

    return _create_validator



import pytest
from common import SCHEMAS_PATH, list_latest_schema_paths, list_schema_paths


@pytest.mark.parametrize("path", list_schema_paths(SCHEMAS_PATH / "unit"))
def test_unit(path, assert_schema_correct):
    assert_schema_correct(path)


@pytest.mark.parametrize("path", list_latest_schema_paths(SCHEMAS_PATH / "unit"))
def test_unit_latest(path, assert_latest_schema_correct):
    assert_latest_schema_correct(path)



import re
from pathlib import Path
from urllib.parse import urljoin

import yaml
from packaging.version import Version

ROOT_PATH = Path(__file__).parent.parent

RESOURCES_PATH = ROOT_PATH / "resources"

SCHEMAS_PATH = RESOURCES_PATH / "schemas" / "stsci.edu" / "asdf"
DOCS_PATH = ROOT_PATH / "docs" / "source"
DOCS_SCHEMAS_PATH = DOCS_PATH / "schemas"
YAML_SCHEMA_PATH = RESOURCES_PATH / "schemas" / "stsci.edu" / "yaml-schema"

MANIFESTS_PATH = RESOURCES_PATH / "manifests" / "asdf-format.org" / "core"

VERSION_MAP_PATHS = list(SCHEMAS_PATH.glob("version_map-*.yaml"))

VALID_YAML_VERSIONS = {"1.1"}
VALID_FILE_FORMAT_VERSIONS = {"1.0.0"}

VALID_SCHEMA_FILENAME_RE = re.compile(r"[a-z0-9_]+-[0-9]+\.[0-9]+\.[0-9]+\.yaml")

DEPRECATED_PATTERNS = {re.compile(".*/transform/.*"), re.compile(".*/wcs/.*")}

METASCHEMA_ID = "http://stsci.edu/schemas/yaml-schema/draft-01"

YAML_TAG_RE = re.compile(r"![a-z/0-9_-]+-[0-9]+\.[0-9]+\.[0-9]")

DESCRIPTION_REF_RE = re.compile(r"\(ref:(.*?)\)")

SCHEMA_ID_PREFIX = "http://stsci.edu/schemas/asdf/"
MANIFEST_ID_PREFIX = "asdf://asdf-format.org/core/manifests/"


def load_yaml(path):
    with path.open() as f:
        return yaml.safe_load(f.read())


def assert_yaml_header_and_footer(path):
    with path.open() as f:
        content = f.read()

    assert any(
        content.startswith(f"%YAML {v}\n---\n") for v in VALID_YAML_VERSIONS
    ), f"{path.name} must start with a %YAML directive with a supported version"

    assert content.endswith("\n...\n"), f"{path.name} must end with '...' followed by a single newline"


def is_deprecated(schema_id_or_tag):
    return any(p.match(schema_id_or_tag) for p in DEPRECATED_PATTERNS)


def split_id(schema_id):
    return schema_id.rsplit("-", 1)


def yaml_tag_to_id(yaml_tag):
    return "http://stsci.edu/schemas/asdf/" + yaml_tag.replace("!", "")


def _relative_stem(path):
    return f"{str((path.parent).relative_to(SCHEMAS_PATH))}/{str(path.stem)}"


def path_to_tag(path):
    return f"tag:stsci.edu:asdf/{_relative_stem(path)}"


def tag_to_path(tag):
    assert tag.startswith("tag:stsci.edu:asdf/")

    return SCHEMAS_PATH / f"""{tag.split("tag:stsci.edu:asdf/")[-1]}.yaml"""


def tag_to_id(tag):
    assert tag.startswith("tag:stsci.edu:asdf/")

    return "http://stsci.edu/schemas/asdf/" + tag.split("tag:stsci.edu:asdf/")[-1]


def id_to_path(id):
    assert id.startswith("http://stsci.edu/schemas/asdf/")

    return SCHEMAS_PATH / f"""{id.split("http://stsci.edu/schemas/asdf/")[-1]}.yaml"""


def path_to_id(path):
    return f"http://stsci.edu/schemas/asdf/{_relative_stem(path)}"


def list_schema_paths(path):
    return sorted(p for p in path.glob("**/*.yaml") if not p.name.startswith("version_map-"))


def list_latest_schema_paths(path):
    paths = list_schema_paths(path)

    latest_by_id_base = {}
    for path in paths:
        schema_id = path_to_id(path)
        id_base, version = split_id(schema_id)
        if id_base in latest_by_id_base:
            if Version(version) > Version(latest_by_id_base[id_base][0]):
                latest_by_id_base[id_base] = (version, path)
        else:
            latest_by_id_base[id_base] = (version, path)

    return sorted([p for _, p in latest_by_id_base.values()])


def list_legacy_schema_paths(path):
    paths = list_schema_paths(path)
    latest_paths = list_latest_schema_paths(path)

    return sorted([p for p in paths if p not in latest_paths])


def ref_to_id(schema_id, ref):
    return urljoin(schema_id, ref)


def list_refs(schema):
    refs = []
    if isinstance(schema, dict):
        for key, value in schema.items():
            if key == "$ref":
                refs.append(value)
            elif isinstance(value, dict) or isinstance(value, list):
                refs.extend(list_refs(value))
    elif isinstance(schema, list):
        for elem in schema:
            refs.extend(list_refs(elem))
    return refs


def list_example_ids(schema):
    if "examples" in schema:
        example_yaml_tags = set()
        for example in schema["examples"]:
            if len(example) == 1:
                example = example[0]
            elif len(example) == 2:
                example = example[1]
            elif len(example) > 2:
                example = example[2]
            else:
                raise RuntimeError(f"Invalid example: {example}")
            example_yaml_tags.update(YAML_TAG_RE.findall(example))
        return sorted({yaml_tag_to_id(yaml_tag) for yaml_tag in example_yaml_tags})
    else:
        return []


def list_description_ids(schema):
    result = set()
    if "description" in schema:
        for ref in DESCRIPTION_REF_RE.findall(schema["description"]):
            if not ref.startswith("http:"):
                ref = "http://stsci.edu/schemas/asdf/" + ref
            result.add(ref)
    return result



import pytest
from common import SCHEMAS_PATH, assert_yaml_header_and_footer, load_yaml
from jsonschema import ValidationError


@pytest.mark.parametrize("path", SCHEMAS_PATH.glob("asdf-schema-*.yaml"))
def test_asdf_schema(path):
    assert_yaml_header_and_footer(path)

    # Asserting no exceptions here
    load_yaml(path)


@pytest.mark.parametrize("path", SCHEMAS_PATH.glob("asdf-schema-*.yaml"))
def test_nested_object_validation(path, create_validator):
    """
    Test that the validations are applied to nested objects.
    """
    metaschema = load_yaml(path)
    validator = create_validator(metaschema)

    schema = {"$schema": metaschema["id"], "type": "object", "properties": {"foo": {"datatype": "float32"}}}
    # No error here
    validator.validate(schema)

    schema = {"$schema": metaschema["id"], "type": "object", "properties": {"foo": {"datatype": "banana"}}}
    with pytest.raises(ValidationError, match="'banana' is not valid"):
        validator.validate(schema)

    schema = {
        "$schema": metaschema["id"],
        "type": "array",
        "items": {"type": "object", "properties": {"foo": {"ndim": "twelve"}}},
    }
    with pytest.raises(ValidationError):
        validator.validate(schema)



from pathlib import Path

import asdf
import pytest
import yaml


def get_resources():
    resources_root = Path(__file__).parent.parent / "resources"

    return {str(path.relative_to(resources_root)): path for path in resources_root.glob("**/*.yaml")}


RESOURCES = get_resources()


@pytest.mark.parametrize("resource", RESOURCES)
def test_resource(resource):
    resource_path = RESOURCES[resource]
    resource_manager = asdf.get_config().resource_manager

    with resource_path.open("rb") as f:
        resource_content = f.read()
    resource = yaml.safe_load(resource_content)

    if "version_map" not in str(resource_path.stem):
        resource_uri = resource["id"]
        assert resource_manager[resource_uri] == resource_content


def get_manifests():
    manifests_root = Path(__file__).parent.parent / "resources" / "manifests" / "asdf-format.org"

    return {str(path.relative_to(manifests_root)): path for path in manifests_root.glob("**/*.yaml")}


MANIFESTS = get_manifests()


@pytest.mark.parametrize("manifest", MANIFESTS)
def test_manifest(manifest):
    manifest_path = MANIFESTS[manifest]
    resource_manager = asdf.get_config().resource_manager

    with manifest_path.open("rb") as f:
        manifest_content = f.read()
    manifest = yaml.safe_load(manifest_content)

    manifest_schema = asdf.schema.load_schema("asdf://asdf-format.org/core/schemas/extension_manifest-1.0.0")

    # The manifest must be valid against its own schema:
    asdf.schema.validate(manifest, schema=manifest_schema)

    for tag_definition in manifest["tags"]:
        # The tag's schema must be available:
        assert tag_definition["schema_uri"] in resource_manager



#!/usr/bin/env python

import os
import re
import subprocess as sp
import sys


def get_schemas(pattern):

    cmd = ["git", "grep", "--name-only"]
    output = sp.check_output(cmd + [pattern, "--", "schemas"]).decode("utf8")
    names = output.split()
    print(names)

    dedupe = dict()

    for name in names:
        version = re.findall(r"\d\.\d.\d", name)[0]
        basepath = name.split("-")[0]
        if basepath in dedupe and dedupe[basepath] > version:
            continue

        dedupe[basepath] = version

    return ["{}-{}.yaml".format(x, y) for x, y in dedupe.items()]


def update_version(string):

    groups = re.search(r"((\d)\.(\d)\.(\d))", string).groups()
    bumped = int(groups[2]) + 1

    new_version = "{}.{}.{}".format(groups[1], bumped, groups[3])
    return re.sub(r"((\d)\.(\d)\.(\d))", new_version, string)


def create_updated_schema(schema, pattern, new_pattern):

    name = os.path.splitext(os.path.basename(schema))[0]
    updated = update_version(name)
    new_schema = re.sub(name, updated, schema)

    with open(new_schema, "w") as new_file:
        with open(schema, "r") as old_file:
            for line in old_file:
                line = line.replace(pattern, new_pattern)
                line = line.replace(name, updated)
                new_file.write(line)


def main():

    if len(sys.argv) != 2:
        name = os.path.basename(sys.argv[0])
        sys.stderr.write("USAGE: {} <pattern>\n".format(name))
        exit(1)

    pattern = sys.argv[1]
    new_pattern = update_version(pattern)
    schemas = get_schemas(pattern)

    for s in schemas:
        create_updated_schema(s, pattern, new_pattern)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

import datetime
import sys
from pathlib import Path

import toml
from pkg_resources import get_distribution

# -- Project information -----------------------------------------------------
try:
    from sphinx_astropy.conf.v1 import *  # noqa
except ImportError:
    print("ERROR: the documentation requires the sphinx-astropy package to be installed")
    sys.exit(1)

# Get configuration information from `pyproject.toml`
with open(Path(__file__).parent.parent.parent / "pyproject.toml") as configuration_file:
    conf = toml.load(configuration_file)
configuration = conf["project"]

project = configuration["name"]
author = f"{configuration['authors'][0]['name']} <{configuration['authors'][0]['email']}>"
copyright = f"{datetime.datetime.now().year}, {configuration['authors'][0]}"

# The short X.Y version
with open(Path(__file__).parent.parent.parent / "README.md") as readme:
    version = readme.readline().split(" ")[-1]

# The full version, including alpha/beta/rc tags
release = get_distribution(configuration["name"]).version

intersphinx_mapping["asdf"] = ("https://asdf.readthedocs.io/en/latest/", None)  # noqa

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions += ["sphinx.ext.mathjax", "sphinx.ext.ifconfig", "sphinx.ext.graphviz", "sphinx_asdf"]  # noqa

mathjax_path = "https://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# The name of the Pygments (syntax highlighting) style to use.
# pygments_style = "sphinx"

# The reST default role (used for this markup: `text`) to use for all documents.
default_role = "ref"

# The default language for highlighting
highlight_language = "yaml"

# The encoding of source files.
source_encoding = "utf-8"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# html_theme_path = sphinx_bootstrap_theme.get_html_theme_path()

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = sphinx_bootstrap_theme.get_html_theme_path()

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "_static/logo.ico"
html_logo = "_static/logo.png"

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = "%b %d, %Y"

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# If false, no module index is generated.
html_domain_indices = False

# If false, no index is generated.
html_use_index = True

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = True

html_context = {"bootswatch_css_custom": ["_static/asdfstandard.css"]}


# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "ASDFStandard"


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    "papersize": "letterpaper",
    # The font size ('10pt', '11pt' or '12pt').
    "pointsize": "10pt",
    # Additional stuff for the LaTeX preamble.
    "preamble": r"""
    % Use a more modern-looking monospace font
    \usepackage[medium]{cabin}
    \usepackage[bitstream-charter]{mathdesign}
    \usepackage{inconsolata}
    \usepackage{bbding}
    \usepackage{fixltx2e}
    \usepackage{microtype}
    \usepackage{changepage}
    \usepackage{enumitem}
    \usepackage{pmboxdraw}
    \MakeRobust\marginpar

    \DeclareUnicodeCharacter{2264}{$\leq$}
    \DeclareUnicodeCharacter{2265}{$\geq$}
    \DeclareUnicodeCharacter{2014}{$\textemdash\textemdash\textemdash$}

    \setlistdepth{10}

    \makeatletter
        \def\marginparright{\@mparswitchfalse}
        \def\marginparoutside{\@mparswitchtrue}
    \makeatother

    \renewenvironment{quote}{\begin{adjustwidth}{15pt}{}}{\end{adjustwidth}}

    """,
    "fontpkg": "",
    "fncychap": "\\usepackage[Conny]{fncychap}",
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, f"ASDFStandard-{version}.tex", "ASDF Standard", author, "manual"),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
latex_logo = "_static/logo.pdf"

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
latex_use_parts = False

# If true, show page references after internal links.
latex_show_pagerefs = True

# If true, show URL addresses after external links.
latex_show_urls = "inline"

# If false, no module index is generated.
latex_domain_indices = False

# Additional latex styles needed
latex_additional_files = ["eqparbox.sty", "capt-of.sty"]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "asdf-standard", project, [author], 1)]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, "ASDFStandard", project, author, "ASDFStandard", "Advanced Scientific Data Format", "Miscellaneous"),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = "Space Telescope Science Institute"
epub_copyright = copyright

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]


# -- sphinx_asdf configuration ---------------------------------------------

# Top-level directory containing ASDF schemas (relative to current directory)
asdf_schema_path = "../../resources/schemas"
# This is the prefix common to all schema IDs in this repository
asdf_schema_standard_prefix = "stsci.edu/asdf"
# This seems pretty redundant/unnecessary and should probably be fixed in the
# plugin.
asdf_schema_reference_mappings = [
    ("tag:stsci.edu:asdf", "http://asdf-standard.readthedocs.io/en/latest/generated/stsci.edu/asdf"),
    ("http://json-schema.org/draft-04/schema", "http://json-schema.org/draft-04/json-schema-validation"),
]


def setup(app):
    app.add_css_file("custom.css")



import fnmatch
import os
from collections.abc import Mapping
from pathlib import Path

__all__ = ["DirectoryResourceMapping"]


class DirectoryResourceMapping(Mapping):
    """
    Resource mapping that reads resource content
    from a directory or directory tree.

    Parameters
    ----------
    root : str or importlib.abc.Traversable
        Root directory (or directory-like Traversable) of the resource
        files.  ``str`` will be interpreted as a filesystem path.
    uri_prefix : str
        Prefix used to construct URIs from file paths.  The
        prefix will be prepended to paths relative to the root
        directory.
    recursive : bool, optional
        If ``True``, recurse into subdirectories.  Defaults to ``False``.
    filename_pattern : str, optional
        Glob pattern that identifies relevant filenames.
        Defaults to ``"*.yaml"``.
    stem_filename : bool, optional
        If ``True``, remove the filename's extension when
        constructing its URI.
    """

    def __init__(self, root, uri_prefix, recursive=False, filename_pattern="*.yaml", stem_filename=True):
        self._uri_to_file = {}
        self._recursive = recursive
        self._filename_pattern = filename_pattern
        self._stem_filename = stem_filename

        if isinstance(root, str):
            self._root = Path(root)
        else:
            self._root = root

        if uri_prefix.endswith("/"):
            self._uri_prefix = uri_prefix[:-1]
        else:
            self._uri_prefix = uri_prefix

        for file, path_components in self._iterate_files(self._root, []):
            self._uri_to_file[self._make_uri(file, path_components)] = file

    def _iterate_files(self, directory, path_components):
        for obj in directory.iterdir():
            if obj.is_file() and fnmatch.fnmatch(obj.name, self._filename_pattern):
                yield obj, path_components
            elif obj.is_dir() and self._recursive:
                yield from self._iterate_files(obj, path_components + [obj.name])

    def _make_uri(self, file, path_components):
        if self._stem_filename:
            filename = os.path.splitext(file.name)[0]
        else:
            filename = file.name

        return "/".join([self._uri_prefix] + path_components + [filename])

    def __getitem__(self, uri):
        return self._uri_to_file[uri].read_bytes()

    def __len__(self):
        return len(self._uri_to_file)

    def __iter__(self):
        yield from self._uri_to_file

    def __repr__(self):
        return "{}({!r}, {!r}, recursive={!r}, filename_pattern={!r}, stem_filename={!r})".format(
            self.__class__.__name__,
            self._root,
            self._uri_prefix,
            self._recursive,
            self._filename_pattern,
            self._stem_filename,
        )



import io
import sys
from collections.abc import Mapping
from pathlib import Path

if sys.version_info < (3, 9):
    import importlib_resources as importlib
else:
    import importlib

from asdf_standard import DirectoryResourceMapping


def test_directory_resource_mapping(tmpdir):
    tmpdir.mkdir("schemas")
    (tmpdir / "schemas").mkdir("nested")
    with (tmpdir / "schemas" / "foo-1.2.3.yaml").open("w") as f:
        f.write("id: http://somewhere.org/schemas/foo-1.2.3\n")
    with (tmpdir / "schemas" / "nested" / "bar-4.5.6.yaml").open("w") as f:
        f.write("id: http://somewhere.org/schemas/nested/bar-4.5.6\n")
    with (tmpdir / "schemas" / "baz-7.8.9").open("w") as f:
        f.write("id: http://somewhere.org/schemas/baz-7.8.9\n")

    mapping = DirectoryResourceMapping(str(tmpdir / "schemas"), "http://somewhere.org/schemas")
    assert isinstance(mapping, Mapping)
    assert len(mapping) == 1
    assert set(mapping) == {"http://somewhere.org/schemas/foo-1.2.3"}
    assert "http://somewhere.org/schemas/foo-1.2.3" in mapping
    assert b"http://somewhere.org/schemas/foo-1.2.3" in mapping["http://somewhere.org/schemas/foo-1.2.3"]
    assert "http://somewhere.org/schemas/baz-7.8.9" not in mapping
    assert "http://somewhere.org/schemas/baz-7.8" not in mapping
    assert "http://somewhere.org/schemas/foo-1.2.3.yaml" not in mapping
    assert "http://somewhere.org/schemas/nested/bar-4.5.6" not in mapping

    mapping = DirectoryResourceMapping(str(tmpdir / "schemas"), "http://somewhere.org/schemas", recursive=True)
    assert len(mapping) == 2
    assert set(mapping) == {"http://somewhere.org/schemas/foo-1.2.3", "http://somewhere.org/schemas/nested/bar-4.5.6"}
    assert "http://somewhere.org/schemas/foo-1.2.3" in mapping
    assert b"http://somewhere.org/schemas/foo-1.2.3" in mapping["http://somewhere.org/schemas/foo-1.2.3"]
    assert "http://somewhere.org/schemas/baz-7.8.9" not in mapping
    assert "http://somewhere.org/schemas/baz-7.8" not in mapping
    assert "http://somewhere.org/schemas/nested/bar-4.5.6" in mapping
    assert b"http://somewhere.org/schemas/nested/bar-4.5.6" in mapping["http://somewhere.org/schemas/nested/bar-4.5.6"]

    mapping = DirectoryResourceMapping(
        str(tmpdir / "schemas"),
        "http://somewhere.org/schemas",
        recursive=True,
        filename_pattern="baz-*",
        stem_filename=False,
    )

    assert len(mapping) == 1
    assert set(mapping) == {"http://somewhere.org/schemas/baz-7.8.9"}
    assert "http://somewhere.org/schemas/foo-1.2.3" not in mapping
    assert "http://somewhere.org/schemas/baz-7.8.9" in mapping
    assert b"http://somewhere.org/schemas/baz-7.8.9" in mapping["http://somewhere.org/schemas/baz-7.8.9"]
    assert "http://somewhere.org/schemas/nested/bar-4.5.6" not in mapping

    # Check that the repr is reasonable
    # Need to be careful checking the path string because
    # pathlib normalizes Windows paths.
    assert repr(Path(str(tmpdir / "schemas"))) in repr(mapping)
    assert "http://somewhere.org/schemas" in repr(mapping)
    assert "recursive=True" in repr(mapping)
    assert "filename_pattern='baz-*'" in repr(mapping)
    assert "stem_filename=False" in repr(mapping)

    # Make sure trailing slash is handled correctly
    mapping = DirectoryResourceMapping(str(tmpdir / "schemas"), "http://somewhere.org/schemas/")
    assert len(mapping) == 1
    assert set(mapping) == {"http://somewhere.org/schemas/foo-1.2.3"}
    assert "http://somewhere.org/schemas/foo-1.2.3" in mapping
    assert b"http://somewhere.org/schemas/foo-1.2.3" in mapping["http://somewhere.org/schemas/foo-1.2.3"]


def test_directory_resource_mapping_with_traversable():
    """
    Confirm that DirectoryResourceMapping doesn't use pathlib.Path
    methods outside of the Traversable interface.
    """

    class MockTraversable(importlib.abc.Traversable):
        def __init__(self, name, value):
            self._name = name
            self._value = value

        def iterdir(self):
            if isinstance(self._value, dict):
                for key, child in self._value.items():
                    yield MockTraversable(key, child)

        def read_bytes(self):
            if not isinstance(self._value, bytes):
                raise RuntimeError("Not a file")
            return self._value

        def read_text(self, encoding="utf-8"):
            return self.read_bytes().decode(encoding)

        def is_dir(self):
            return isinstance(self._value, dict)

        def is_file(self):
            return self._value is not None and not isinstance(self._value, dict)

        def joinpath(self, child):
            if isinstance(self._value, dict):
                child_value = self._value.get(child)
            else:
                child_value = None

            return MockTraversable(child, child_value)

        def __truediv__(self, child):
            return self.joinpath(child)

        def open(self, mode="r", *args, **kwargs):
            if not self.is_file():
                raise RuntimeError("Not a file")

            if mode == "r":
                return io.TextIOWrapper(io.BytesIO(self._value), *args, **kwargs)
            elif mode == "rb":
                return io.BytesIO(self._value)
            else:
                raise "Not a valid mode"

        @property
        def name(self):
            return self._name

    root = MockTraversable(
        "/path/to/some/root",
        {"foo-1.0.0.yaml": b"foo", "bar-1.0.0.yaml": b"bar", "baz-1.0.0": b"baz", "nested": {"foz-1.0.0.yaml": b"foz"}},
    )

    mapping = DirectoryResourceMapping(root, "http://somewhere.org/schemas")
    assert len(mapping) == 2
    assert set(mapping) == {"http://somewhere.org/schemas/foo-1.0.0", "http://somewhere.org/schemas/bar-1.0.0"}
    assert "http://somewhere.org/schemas/foo-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/foo-1.0.0"] == b"foo"
    assert "http://somewhere.org/schemas/bar-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/bar-1.0.0"] == b"bar"
    assert "http://somewhere.org/schemas/baz-1.0.0" not in mapping
    assert "http://somewhere.org/schemas/nested/foz-1.0.0" not in mapping

    mapping = DirectoryResourceMapping(root, "http://somewhere.org/schemas", recursive=True)
    assert len(mapping) == 3
    assert set(mapping) == {
        "http://somewhere.org/schemas/foo-1.0.0",
        "http://somewhere.org/schemas/bar-1.0.0",
        "http://somewhere.org/schemas/nested/foz-1.0.0",
    }
    assert "http://somewhere.org/schemas/foo-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/foo-1.0.0"] == b"foo"
    assert "http://somewhere.org/schemas/bar-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/bar-1.0.0"] == b"bar"
    assert "http://somewhere.org/schemas/baz-1.0.0" not in mapping
    assert "http://somewhere.org/schemas/nested/foz-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/nested/foz-1.0.0"] == b"foz"

    mapping = DirectoryResourceMapping(
        root, "http://somewhere.org/schemas", filename_pattern="baz-*", stem_filename=False
    )
    assert len(mapping) == 1
    assert set(mapping) == {"http://somewhere.org/schemas/baz-1.0.0"}
    assert "http://somewhere.org/schemas/foo-1.0.0" not in mapping
    assert "http://somewhere.org/schemas/bar-1.0.0" not in mapping
    assert "http://somewhere.org/schemas/baz-1.0.0" in mapping
    assert mapping["http://somewhere.org/schemas/baz-1.0.0"] == b"baz"
    assert "http://somewhere.org/schemas/nested/foz-1.0.0" not in mapping



import pytest
from common import YAML_SCHEMA_PATH, assert_yaml_header_and_footer, list_schema_paths, load_yaml
from jsonschema import ValidationError


@pytest.mark.parametrize("path", list_schema_paths(YAML_SCHEMA_PATH))
def test_yaml_schema(path):
    assert_yaml_header_and_footer(path)

    # Asserting no exceptions here
    load_yaml(path)


@pytest.mark.parametrize("path", YAML_SCHEMA_PATH.glob("*.yaml"))
def test_nested_object_validation(path, create_validator):
    """
    Test that the validations are applied to nested objects.
    """
    metaschema = load_yaml(path)
    validator = create_validator(metaschema)

    schema = {"$schema": metaschema["id"], "type": "object", "properties": {"foo": {"flowStyle": "block"}}}
    # No error here
    validator.validate(schema)

    schema = {"$schema": metaschema["id"], "type": "object", "properties": {"foo": {"flowStyle": "funky"}}}
    with pytest.raises(ValidationError, match="'funky' is not one of"):
        validator.validate(schema)

    schema = {
        "$schema": metaschema["id"],
        "type": "array",
        "items": {"type": "object", "properties": {"foo": {"propertyOrder": "a,b,c,d"}}},
    }
    with pytest.raises(ValidationError):
        validator.validate(schema)



import asdf
import pytest
from common import MANIFESTS_PATH, RESOURCES_PATH, load_yaml

MANIFEST_PATHS = sorted(MANIFESTS_PATH.glob("*.yaml"))
MANIFEST_SCHEMA_PATH = RESOURCES_PATH / "schemas" / "asdf-format.org" / "core" / "extension_manifest-1.0.0.yaml"
MANIFEST_SCHEMA_ID = "asdf://asdf-format.org/core/schemas/extension_manifest-1.0.0"


@pytest.mark.parametrize("path", MANIFEST_PATHS)
def test_manifest(path, tag_to_schema):
    manifest = load_yaml(path)

    with asdf.config_context() as config:
        config.add_resource_mapping({MANIFEST_SCHEMA_ID: MANIFEST_SCHEMA_PATH.read_bytes()})

        manifest_schema = asdf.schema.load_schema(MANIFEST_SCHEMA_ID)
        asdf.schema.validate(manifest, schema=manifest_schema)

    assert "title" in manifest
    assert "description" in manifest

    for tag in manifest["tags"]:
        if (
            "time" not in tag["tag_uri"]
            and "core" not in tag["tag_uri"]
            and "unit" not in tag["tag_uri"]
            and "wcs" not in tag["tag_uri"]
        ):
            assert tag["tag_uri"] in tag_to_schema
            schema = tag_to_schema[tag["tag_uri"]][0]
            assert tag["schema_uri"] == schema["id"]
        assert "title" in tag
        assert "description" in tag



import re

import pytest
from common import (
    VALID_FILE_FORMAT_VERSIONS,
    VALID_YAML_VERSIONS,
    VERSION_MAP_PATHS,
    assert_yaml_header_and_footer,
    is_deprecated,
    load_yaml,
    path_to_id,
    split_id,
)
from packaging.version import Version

VALID_FILENAME_RE = re.compile(r"version_map-[0-9]+\.[0-9]+\.[0-9]+\.yaml")

SORTED_PATHS = sorted(VERSION_MAP_PATHS, key=lambda p: Version(split_id(path_to_id(p))[1]))

LATEST_PATH = SORTED_PATHS[-1]


@pytest.mark.parametrize("path", VERSION_MAP_PATHS)
def test_version_map(path, schema_tags):
    assert VALID_FILENAME_RE.match(path.name) is not None, f"{path.name} is an invalid version map filename"

    assert_yaml_header_and_footer(path)

    vm = load_yaml(path)

    assert set(vm.keys()) == {"FILE_FORMAT", "YAML_VERSION", "tags"}

    assert vm["FILE_FORMAT"] in VALID_FILE_FORMAT_VERSIONS
    assert vm["YAML_VERSION"] in VALID_YAML_VERSIONS

    for tag_base, tag_version in vm["tags"].items():
        tag = f"{tag_base}-{tag_version}"
        if "time" not in tag and "core" not in tag and "unit" not in tag and "wcs" not in tag:
            assert tag in schema_tags, f"{path.name} specifies missing tag {tag}"

    assert len(vm["tags"].keys()) == len(set(vm["tags"].keys())), f"{path.name} contains duplicate tags"

    sorted_tags = sorted(list(vm["tags"].keys()))
    if sorted_tags != list(vm["tags"].keys()):
        sorted_list = "\n".join([f"""{tag}: {vm["tags"][tag]}""" for tag in sorted_tags])
        message = f"{path.name} tag list is not sorted.  Try this order instead:\n{sorted_list}"
        assert False, message


def test_latest_version_map(latest_schema_tags):
    """
    The current latest version map has some special requirements.
    """
    vm = load_yaml(LATEST_PATH)

    tag_base_to_version = dict([tag.rsplit("-", 1) for tag in latest_schema_tags])

    expected_tag_bases = set([t for t in tag_base_to_version.keys() if not is_deprecated(t)])
    vm_tag_bases = set(vm["tags"].keys())
    if not expected_tag_bases.issubset(vm_tag_bases):
        missing_tag_bases = expected_tag_bases - vm_tag_bases
        insert_list = "\n".join(
            sorted(f"""{tag_base}-{tag_base_to_version[tag_base]}""" for tag_base in missing_tag_bases)
        )
        message = (
            f"{LATEST_PATH.name} must include the latest version of "
            "every non-deprecated schema with a tag.  Update the deprecation "
            "list in tests/common.py, or add the following missing schemas: \n"
            f"{insert_list}"
        )
        assert False, message

    incorrect_tag_bases = sorted([tag for tag in expected_tag_bases if vm["tags"][tag] != tag_base_to_version[tag]])
    if len(incorrect_tag_bases) > 0:
        update_list = "\n".join(
            [f"""{tag}: {vm["tags"][tag]} --> {tag_base_to_version[tag]}""" for tag in incorrect_tag_bases]
        )
        message = (
            f"{LATEST_PATH.name} must include the latest version of "
            "every non-deprecated schema with a tag.  Update the following: \n"
            f"{update_list}"
        )
        assert False, message


@pytest.mark.parametrize("path, previous_path", zip(SORTED_PATHS[1:], SORTED_PATHS[0:-1]))
def test_version_map_tags_retained(path, previous_path):
    """
    Confirm that non-deprecated tags were not lost between
    successive version maps.
    """
    vm = load_yaml(path)
    prev_vm = load_yaml(previous_path)

    expected_tags = set([t for t in prev_vm["tags"].keys() if not is_deprecated(t)])
    tags = set(vm["tags"].keys())
    if not expected_tags.issubset(tags):
        missing_tags = expected_tags - tags
        assert False, (
            f"{path.name} is missing schemas that were present in the "
            "previous version.  If this was intentional, update the deprecation "
            "list in tests/common.py, otherwise add the following missing schemas: "
            f"""{", ".join(missing_tags)}"""
        )



import collections

from common import is_deprecated

EXCEPTIONS = {
    "tag:stsci.edu:asdf/asdf-schema-1.0.0",
    "asdf://asdf-format.org/core/manifests/:schema_root: ../../resources/manifests",
    "asdf://asdf-format.org/core/manifests/:standard_prefix: asdf-format.org/core",
}


def test_docs_schema_links(
    latest_schema_ids, legacy_schema_ids, manifest_ids, docs_schema_ids, docs_legacy_schema_ids, docs_manifest_ids
):
    """
    Confirm that the latest versions of all non-deprecated schemas
    are represented in the documentation.
    """
    expected_schema_ids = set()
    for schema_id in latest_schema_ids:
        if not is_deprecated(schema_id):
            expected_schema_ids.add(schema_id)

    extra_legacy_docs_ids = set(docs_legacy_schema_ids) - legacy_schema_ids - EXCEPTIONS
    if len(extra_legacy_docs_ids) > 0:
        remove_list = "\n".join(sorted(list(extra_legacy_docs_ids)))
        message = (
            "Only the legacy documentation should link to old versions of schemas. "
            "Remove or update the following links: \n"
            f"{remove_list}"
        )
        assert False, message

    extra_docs_ids = set(docs_schema_ids) - expected_schema_ids - EXCEPTIONS - set(docs_legacy_schema_ids)
    if len(extra_docs_ids) > 0:
        remove_list = "\n".join(sorted(list(extra_docs_ids)))
        message = (
            "The main documentation should not link to old versions of schemas. "
            "Remove or update the following links to the legacy documentation: \n"
            f"{remove_list}"
        )
        assert False, message

    missing_docs_ids = expected_schema_ids - set(docs_schema_ids) - EXCEPTIONS
    if len(missing_docs_ids) > 0:
        insert_list = "\n".join(sorted(list(missing_docs_ids)))
        message = (
            "The documentation must include a link to the latest version of "
            "every non-deprecated schema with a tag.  Update the deprecation "
            "list in tests/common.py, or add links to the following missing schemas: \n"
            f"{insert_list}"
        )
        assert False, message

    missing_manifest_docs_ids = manifest_ids - set(docs_manifest_ids) - EXCEPTIONS
    if len(missing_manifest_docs_ids) > 0:
        print(missing_manifest_docs_ids)
        insert_list = "\n".join(sorted(list(missing_manifest_docs_ids)))
        message = (
            "The documentation must include a link to every version of the asdf-standard"
            "manifest. Please add the following links to the documentation: \n"
            f"{insert_list}"
        )
        assert False, message

    counter = collections.Counter(docs_schema_ids)
    if max(counter.values()) > 1:
        remove_list = "\n".join(sorted([tag for tag, count in counter.items() if count > 1]))
        message = "The documentation contains duplicate links to the following schemas: \n" f"{remove_list}"
        assert False, message



# Generate the file in resources/asdf-format.org/core/manifests/

import glob
import os

import yaml

os.chdir(os.path.join(os.path.dirname(__file__), ".."))

SCHEMA_PATTERNS = [
    "schemas/stsci.edu/asdf/core/*.yaml",
    "schemas/stsci.edu/asdf/fits/*.yaml",
    "schemas/stsci.edu/asdf/time/*.yaml",
    "schemas/stsci.edu/asdf/unit/*.yaml",
    "schemas/stsci.edu/asdf/wcs/*.yaml",
]


def represent_string(dumper, data):
    if len(data.splitlines()) > 1:
        style = "|"
    else:
        style = None

    return dumper.represent_scalar("tag:yaml.org,2002:str", data, style=style)


yaml.SafeDumper.add_representer(str, represent_string)

schemas_by_tag = {}
for pattern in SCHEMA_PATTERNS:
    for path in glob.glob(pattern):
        schema = yaml.safe_load(open(path).read())
        tag = "tag:stsci.edu:" + schema["id"].split("http://stsci.edu/schemas/")[-1]
        schemas_by_tag[tag] = schema

for path in sorted(glob.glob("schemas/stsci.edu/asdf/version_map-*.yaml")):
    version_map = yaml.safe_load(open(path).read())
    version = path.split("/")[-1].split("-")[-1].split(".yaml")[0]
    tags = sorted([k + "-" + str(v) for k, v in version_map["tags"].items() if "/transform/" not in k])

    tag_defs = []
    for tag in tags:
        schema = schemas_by_tag[tag]
        tag_def = {
            "tag_uri": tag,
            "schema_uri": schema["id"],
            "title": schema["title"].strip(),
            "description": schema["description"].strip(),
        }
        tag_defs.append(tag_def)

    manifest = {
        "id": f"asdf://asdf-format.org/core/manifests/core-{version}",
        "extension_uri": f"asdf://asdf-format.org/core/extensions/core-{version}",
        "title": f"Core extension {version}",
        "description": "Tags for ASDF core objects.",
        "asdf_standard_requirement": version,
        "tags": tag_defs,
    }

    with open(f"resources/asdf-format.org/core/manifests/core-{version}.yaml", "w") as f:
        yaml.dump(manifest, f, sort_keys=False, Dumper=yaml.SafeDumper)



import sys
from pathlib import Path

if sys.version_info < (3, 9):
    import importlib_resources
else:
    import importlib.resources as importlib_resources

import asdf_standard


def get_resource_mappings():
    resources_root = importlib_resources.files(asdf_standard) / "resources"
    if not resources_root.is_dir():
        # In an editable install, the resources directory will exist off the
        # repository root:
        resources_root = Path(__file__).absolute().parent.parent.parent / "resources"
        if not resources_root.is_dir():
            raise RuntimeError("Missing resources directory")

    return [
        asdf_standard.DirectoryResourceMapping(
            resources_root / "schemas" / "stsci.edu", "http://stsci.edu/schemas/", recursive=True
        ),
        asdf_standard.DirectoryResourceMapping(
            resources_root / "schemas" / "asdf-format.org" / "core", "asdf://asdf-format.org/core/schemas/"
        ),
        asdf_standard.DirectoryResourceMapping(
            resources_root / "manifests" / "asdf-format.org" / "core",
            "asdf://asdf-format.org/core/manifests/",
        ),
    ]
